/*  
 * main.cpp
 *
 * This file is part of qmol - A simple molecular weight calculator
 *
 * Copyright (C) 2012 Thomas Mitterfellner
 * Copyright (C) 2009 Tomislav Gountchev
 *
 */

/***************************************************************************
 *                                                                         *
 *   qmol is free software; you can redistribute it and/or modify          *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful, but   *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA 02111-1307 USA                                                     *
 *                                                                         *
 ***************************************************************************/

#include <QtWidgets/QApplication>
#include <QMessageBox>
#include <QSettings>
#include <QDir>
#include <QStandardPaths>
#include <QTextStream>
#include <QTranslator>
#include <QLibraryInfo>
#include <QDebug>

#include "qmol.h"
#include "qmolcalc.h"
#include "stdio.h"

#ifdef Q_OS_UNIX
  #include "cmake_defines.h" // this is generated by cmake and defines the installation path
#endif

#ifdef Q_OS_WIN
  #define INSTALLATION_PATH ""
  #define _WIN32_WINNT 0x0501
  #include <windows.h>
#endif

int main(int argc, char *argv[])
{
    QApplication a(argc, argv);
    QString installationPath = INSTALLATION_PATH;

    //QCoreApplication::setOrganizationName("qmol");
    //QCoreApplication::setOrganizationDomain("qmol");
    QCoreApplication::setApplicationName("qmol");

    QString dataPath = QStandardPaths::standardLocations(QStandardPaths::DataLocation).join("");
    if (!QDir(dataPath).exists()) QDir().mkdir(dataPath);

    QTranslator qtTranslator;
    qtTranslator.load("qt_" + QLocale::system().name(),
            QLibraryInfo::location(QLibraryInfo::TranslationsPath));
    a.installTranslator(&qtTranslator);

    QTranslator qmolTranslator;
    qmolTranslator.load("qmol_" + QLocale::system().name(), installationPath+"./translations");
    a.installTranslator(&qmolTranslator);


    QStringList args = QCoreApplication::arguments();

    if (args.count() > 1) {
        qmolPreferences preferences;
        preferences.restoreSettings();
        ElementTable elements;
        GroupTable groups(&elements, &preferences);
        QString outputString;
        qMolCalc qmolcalc(&elements, &groups, &preferences);
        double formulaWeight = 0;

#ifdef Q_OS_WIN
        // link stdout to the console which is calling qmol.exe
        AttachConsole(ATTACH_PARENT_PROCESS);
        freopen("CONOUT$","wb",stdout);
#endif
        for (int i = 1; i < args.count(); i++) {
            QString inputFormula = QString(args.at(i));
            QString error = qmolcalc.parseFormula(inputFormula);

            QTextStream str(stdout);

#ifdef Q_OS_WIN
            // the windows command prompt seems to need an initial newline
            str << "\n";
#endif
            if (error == "OK") {
                outputString = qmolcalc.getResultsOutput(m_CommandLine, true);

                str << outputString << "\n";
            } else
                str << "ERROR: " << error << "\n\n";
        }
        return 0;
    } else {
        qmol w;
        QObject::connect(&a, SIGNAL(lastWindowClosed()), &w, SLOT(quit()));
        w.show();
        return a.exec();
    }
}
