#include <QColorDialog>
#include <QSettings>
#include <QDebug>

#include "options_dialog.h"
#include "qmolcalc.h"
#include "ui_options_dialog.h"

const QString COLOR_STYLE("QPushButton { background-color : %1 }");
const QString BORDER_RADIUS("QPushButton { border-radius : %1 }");

options_dialog::options_dialog(qmolPreferences* preferences_, QWidget *parent) :
    QDialog(parent),
    ui(new Ui::options_dialog)
{
    preferences = preferences_;

    sampleOutput = new qMolCalc(NULL, NULL, preferences);
    sampleOutput->parseFormula("[DF]Me-H");

    ui->setupUi(this);
    resize(preferences->preferencesSize);
    move(preferences->preferencesPosition);

    ui->pbSubscriptColor->setPalette(QPalette(preferences->coeffColor));
    ui->cbSubScript->setChecked(preferences->subscriptCoeffient);
    ui->cbProteinMode->setChecked(preferences->proteinMode);

    updateSampleFormula();
    ui->leCompositionFormat->setText(preferences->compositionFormat);
    ui->leNumberAA->setText(preferences->numberAAformat);

    ui->leOutputFormat->setText(preferences->outputFormat);
    ui->leOutputFormatGUI->setText(preferences->outputFormatGUI);

    updateCommandLineOutput(preferences->outputFormat);
    updateGUIoutput(preferences->outputFormatGUI);

    ui->pbOK->setFocus();

}

options_dialog::~options_dialog()
{
    delete sampleOutput;
    delete ui;
}

void options_dialog::chooseSubscriptColor() {
    QColor color = QColorDialog::getColor(preferences->coeffColor, this);
    if (color.isValid()) {
        //ui->pbSubscriptColor->setStyleSheet(COLOR_STYLE.arg(color.name()));
        ui->pbSubscriptColor->setPalette(QPalette(color));
        updateSampleFormula();
    }
}

void options_dialog::onCancelClicked() {
    this->close();
}
void options_dialog::onOKclicked() {
    applySettings();
    this->close();
}

void options_dialog::applySettings() {
    preferences->proteinMode = ui->cbProteinMode->isChecked();
    preferences->subscriptCoeffient = ui->cbSubScript->isChecked();
    preferences->coeffColor = ui->pbSubscriptColor->palette().background().color();

    preferences->numberAAformat = ui->leNumberAA->text();
    preferences->compositionFormat = ui->leCompositionFormat->text();

    preferences->outputFormat = ui->leOutputFormat->text();
    preferences->outputFormatGUI = ui->leOutputFormatGUI->text();
    updateCommandLineOutput(preferences->outputFormat);
    updateGUIoutput(preferences->outputFormatGUI);
}

void options_dialog::updateSampleFormula() {
    QColor color = ui->pbSubscriptColor->palette().background().color();
    if (!color.isValid()) {
        color = preferences->coeffColor;
    }
    bool subscriptCoeff = ui->cbSubScript->isChecked();

    QString sumFormulaHTML = qMolCalc::plainFormulaToHTML("C2H5OH", color, subscriptCoeff);

    ui->sampleFormula->setHtml(sumFormulaHTML);
}

void options_dialog::updateCompositionFormat(QString outputFormat){
    preferences->compositionFormat = outputFormat;
    updateCommandLineOutput(ui->leOutputFormat->text());
    updateGUIoutput(ui->leOutputFormatGUI->text());
}

void options_dialog::updateNumberAAformat(QString outputFormat){
    preferences->numberAAformat = outputFormat;
    updateCommandLineOutput(ui->leOutputFormat->text());
    updateGUIoutput(ui->leOutputFormatGUI->text());
}

void options_dialog::updateGUIoutput(QString outputFormat){
    sampleOutput->printFullComposition(ui->teSampleOutputGUI, outputFormat, m_GUI);
}

void options_dialog::updateCommandLineOutput(QString outputFormat){
    sampleOutput->printFullComposition(ui->teSampleOutput, outputFormat, m_CommandLine);
}
