#include <QUrl>
#include <QSettings>
#include <QDesktopServices>
#include <QDesktopServices>

#include "preferences.h"

qmolPreferences::qmolPreferences() {
}

qmolPreferences::~qmolPreferences() {
}

void qmolPreferences::restoreSettings() {
    QSettings settings(QSettings::IniFormat, QSettings::UserScope,
                                    "qmol", "qmol");
    settings.beginGroup("outputFormatting");
        numberAAformat = settings.value("numberAAformat", "Sum of AA: %N\\n").toString();
        compositionFormat = settings.value("compositionFormat", "%E\\t%C %\\n").toString();
        outputFormat = settings.value("outputFormat", "%i = %o:\\n%w g/mol\\n%a%c").toString();
        outputFormatGUI = settings.value("outputFormatGUI", "%i = %o:\\n%a%c").toString();
    settings.endGroup();
    settings.beginGroup("activePredefSets");
        int i = 0;
        foreach (QString predefKey, predefSetsNames) {
            activePredefSets.append(settings.value(predefKey, defaultActivePredefSets[i++]).toBool());
        }
        activePredefSets.replace(m_AminoAcids1L, true);
    settings.endGroup();
}

void qmolPreferences::restoreGUISettings() {
    QSettings settings(QSettings::IniFormat, QSettings::UserScope,
                                    "qmol", "qmol");
    settings.beginGroup("Windows");
        size = settings.value("size", QSize(500, 740)).toSize();
        position = settings.value("pos", QPoint(100, 100)).toPoint();
        preferencesSize = settings.value("preferencesSize", QSize(800, 600)).toSize();
        preferencesPosition = settings.value("preferencesPos", QPoint(500, 100)).toPoint();
        groupDefsSize = settings.value("groupDefsSize", QSize(400, 600)).toSize();
        groupDefsPosition = settings.value("groupDefsPos", QPoint(500, 100)).toPoint();
    settings.endGroup();
    settings.beginGroup("splitterSizes");
        splitterSizes = settings.value("splitterSizes").toByteArray();
    settings.endGroup();
    settings.beginGroup("Preferences");
        proteinMode = settings.value("proteinMode", true).toBool();
        subscriptCoeffient = settings.value("subscriptCoeffs", true).toBool();
        coeffColor = QColor(settings.value("coeffColor", QColor(0, 0, 204).name()).toString());
        reverseHistory = settings.value("reverseHistory", true).toBool();
    settings.endGroup();
}

void qmolPreferences::storeSettings() {
    int i = 0;
    QSettings settings(QSettings::IniFormat, QSettings::UserScope,
                                    "qmol", "qmol");

    settings.beginGroup("Windows");
        settings.setValue("size", size);
        settings.setValue("pos", position);
        settings.setValue("preferencesSize", preferencesSize);
        settings.setValue("preferencesPos", preferencesPosition);
        settings.setValue("groupDefsSize", groupDefsSize);
        settings.setValue("groupDefsPos", groupDefsPosition);
    settings.endGroup();
    settings.beginGroup("Preferences");
        settings.setValue("proteinMode", proteinMode);
        settings.setValue("subscriptCoeffs", subscriptCoeffient);
        settings.setValue("coeffColor", coeffColor);
        settings.setValue("reverseHistory", reverseHistory);
    settings.endGroup();
    settings.beginGroup("splitterSizes");
        settings.setValue("splitterSizes", splitterSizes);
    settings.endGroup();
    settings.beginGroup("outputFormatting");
        settings.setValue("numberAAformat", numberAAformat);
        settings.setValue("compositionFormat", compositionFormat);
        settings.setValue("outputFormat", outputFormat);
        settings.setValue("outputFormatGUI", outputFormatGUI);
    settings.endGroup();
    settings.beginGroup("activePredefSets");
        foreach (QString keyName, predefSetsNames) {
            settings.setValue(keyName, activePredefSets.value(i++));
        }
    settings.endGroup();
}
