/*
 * qmol.h
 *
 * This file is part of qmol - A simple molecular weight calculator
 */

#ifndef QMOL_H
#define QMOL_H

#include <QMainWindow>
#include <QTextEdit>
#include <QItemDelegate>
#include <QStandardItemModel>
#include <QPainter>
#include <QStyledItemDelegate>
#include <QCompleter>

#include "group_definitions.h"
#include "about.h"
#include "preferences.h"

class tableDelegate;

namespace Ui {
    class qmol;
}

class qmol : public QMainWindow
{
    Q_OBJECT

public:
    explicit qmol(QWidget *parent = 0);
    ~qmol();

    qmolPreferences* preferences;

    QSet<QString>* overrides;
    ElementTable* elements;
    GroupTable* groups;

private slots:
    void on_clear_fields_clicked();
    void on_calculate_clicked();
    void on_formula_returnPressed();

    void clearHistory();
    void removeFromHistory();
    void refreshHistory();
    void onSelectionChanged(const QItemSelection &, const QItemSelection &);

    void openDefineDialog();
    void openOptionsDialog();
    void reverseHistory();
    void showAbout();
    void quit();
    void openHandbook();
    void addToHistory(QString);

private:
    Ui::qmol *ui;

    void clear();
    void calc();
    //void printFullComposition(QStringList*, QTextEdit*);
    //void printFullComposition(QString, QTextEdit*);
    void setHistoryVisibility();
    //void resizeEvent(QResizeEvent *event);

    QStandardItemModel* model;
    tableDelegate* delegate;
    QCompleter* completer;
    qMolCalc* qmolcalc;
    QString sumFormulaHTML;
    group_definitions* groupDefsWindow;
    aboutDialog* aboutWindow;
};

class tableDelegate : public QStyledItemDelegate
 {
     Q_OBJECT

 public:
     tableDelegate(QObject *parent = 0);

     QTableView* historyTableView;
     void paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const;
 };

#endif // QMOL_H
