/*
 * qmolcalc.h
 *
 * This file is part of qmol - A simple molecular weight calculator
 *
 * Copyright (C) 2012 Thomas Mitterfellner
 * Copyright (C) 2009 Tomislav Gountchev
 *
 */

#ifndef QMOLCALC_H
#define QMOLCALC_H

#include "qmolelements.h"
#include <QTextEdit>
#include "preferences.h"

const QString c_OK = "OK";
const int m_noBrace = 0; // we're not inside of braces
const int m_squareBracket = 1; // we're inside of []
const int m_parenthesis = 2; // we're inside of ()

const int m_GUI = 0;
const int m_CommandLine = 1;

class qMolCalc {
public:
    qMolCalc(ElementTable* elements_, GroupTable* groups_, qmolPreferences* preferences_=NULL);
    ~qMolCalc();

    static QString plainFormulaToHTML(QString s, QColor coeffColor, bool subscriptCoeff);
    void printFullComposition(QTextEdit* compositionWidget, QString outputFormat_, int outputMode = m_GUI);

    QString parseFormula(const QString& s, Composition* comp = NULL);
    void setOutputFormat(QString outputFormat_);

    bool compIsEmpty() const;
    double getWeight() const;
    double getFormulaWeight() const;
    QString getSumFormula() const;
    QString getSumFormulaHTML() const;
    QString getResultsOutput(int outputMode, bool createResults = false);

    static double calculateFormulaWeight(QString formula, ElementTable* elementTable);
    void clearComp();

    Composition* getComposition();
    QStringList* getFullComposition();

    bool hasNegativeCoefficients();

private:
    void makeSumFormula();
    void makeSumFormulaHTML(QString);
    void makeFullComposition(QString outputFormat_);
    void calcPercentages();
    void calcWeight();

    qmolPreferences* preferences;

    ElementTable* elements;
    GroupTable* groups;
    QString parseString(const QString& s, Composition* comp, bool aaMode = false, int brace = m_noBrace);
    QString checkBrackets(const QString& s, int sl, int* i, int j, QChar bracket);

    /* GroupList* groups;*/
    Composition* composition;
    QString inputFormula; // formula as entered
    QString sumFormula; // sum formula (only elements, without duplicates)

    bool proteinMode;
    bool subscriptCoeff;
    QString coeffColor;
    QString sumFormulaHTML; // sum formula with HTML tags (only elements, without duplicates)
    QList<double>* percent; // percentage values
    QString outputFormat; // string containing the output format for the command line output
    QString compositionOneLine; // elemental composition in one string, each line separated by a newline character
    QString commandLineOutput; // calculation result formatted for command line output
    QString guiOutput; // calculation result formatted for GUI output (HTML for subscripts etc.)
    QStringList* fullComposition; // multiline elemental composition string including percentage values
    double formulaWeight;

    bool deleteElementsTable, deleteGroupsTable;
};

QString eatWhiteSpace(QString);

#endif // QMOLCALC_H
