/*
 * qmolelements.h
 *
 * This file is part of qmol - A simple molecular weight calculator
 *
 * Copyright (C) 2012 Thomas Mitterfellner
 * Copyright (C) 2009 Tomislav Gountchev
 *
 */

#ifndef QMOLELEMENTS_H
#define QMOLELEMENTS_H

#include <QString>
#include <QHash>

#include "preferences.h"

const QString c_overriddensymbol = "!";

class qMolCalc; // Forward decl for friendship
class ElementTable;

class Composition {
public:
    Composition();
    ~Composition();

    friend class qMolCalc; // So it can access list & its methods;

    void addElToList(QString name, double coef);
    //void addToList(QMap<QString, double>* comp);
    void addToList(Composition* comp);
    void clear();
    //void subtractH2O(int number);
    void subtractH2O();
    int getNumberAminoAcids();
    int getNumberProteins();
    void addAminoAcid(int number);
    void addProtein(int number);
    void clearAA();

    void applyCoefficient(double suff, bool multAA = false);
    void setList(QMap<QString, double>* comp);

    QMap<QString, double>* getList();

private:
    QMap<QString, double>* list;
    bool mustDelete;
    double suffix;
    int numberAminoAcids;
    int numberProteins;
    int sign;
};


class UnitTable {
    virtual void readTable() = 0;
};


class ElementTable {
public:
    ElementTable();
    ~ElementTable();

    void readTable();
    double getWeight(QString name) const;
    bool insert(QString name, double weight);
    void remove(QString name);
    bool contains(QString name) const;
    QList<QString>* getElementList();
    void override(const QString name);
    void undoOverride(const QString name);

private:
    QHash<QString, double> elTable;
    //QHash<QString, double> allWeights;
};

class GroupTable {
public:
    GroupTable(ElementTable* elementTable, qmolPreferences* preferences_);
    ~GroupTable();

    friend class qMolCalc; // So it can access list & its methods;
    void readTables();
    void readTable(QString tableName = "");
    double getWeight(QString name) const;
    QString getFormula(QString name) const;
    QString get1LAminoAcid(QString name) const;
    bool contains(QString name) const;
    bool isAA(QString name) const;
    QHash<QString, QString>* getGrpFormulaTable();
    QHash<QString, double>* getGrpWeightTable();
    QList<QString>* getOverriddenTable();
    void overrideUsedElementSymbols();
    void addToOverriding(QString name);
    bool isOverriding(QString name);

private:
    ElementTable* elements;
    QList<QString> predefSetsFileNames;
    QHash<QString, QString> userdefGroups;
    QHash<QString, QString> predefGroups;
    QHash<QString, QString> aminoAcids1L;
    QHash<QString, QString> aminoAcids;
    QHash<QString, QString> allGroups;
    QHash<QString, double> grpWeightTable;
    qmolPreferences* preferences;
    QList<QString> overriddenSymbols; // contains a list of element symbols overriden by groupsymbols
    void addToGroupFormulaTable(QString name, QString compString);
    double weight;
};


#endif // QMOLELEMENTS_H
